/// <reference types="node" />
import { EventEmitter } from 'events';
import { clouddebugger_v2 } from 'googleapis';
export declare type OneIndexedLineNumber = number;
export declare type OneIndexedColumnNumber = number;
export declare type BreakpointId = string;
export declare type DebuggeeId = string;
export declare type DebuggerId = string;
export declare type Expression = string;
export declare type ProjectId = string;
export declare type SourcePath = string;
export declare type Timestamp = string;
export declare type WaitToken = string;
export interface Options {
    debuggerId: DebuggerId;
    sourceDirectory: SourcePath;
}
export declare type Variable = clouddebugger_v2.Schema$Variable;
export interface SourceLocation extends clouddebugger_v2.Schema$SourceLocation {
    path: SourcePath;
    line: OneIndexedLineNumber;
    column?: OneIndexedColumnNumber;
}
export interface StackFrame extends clouddebugger_v2.Schema$StackFrame {
    function: string;
    location: SourceLocation;
}
export declare enum Action {
    CAPTURE = "CAPTURE",
    LOG = "LOG"
}
export interface BreakpointRequest extends clouddebugger_v2.Schema$Breakpoint {
    action: Action;
    location: SourceLocation;
}
export interface Breakpoint extends clouddebugger_v2.Schema$Breakpoint {
    id: BreakpointId;
    location: SourceLocation;
}
export interface CapturedSnapshot extends Breakpoint {
    isFinalState: true;
    stackFrames: StackFrame[];
    variableTable: Variable[];
}
export interface Debuggee extends clouddebugger_v2.Schema$Debuggee {
    id: DebuggeeId;
    labels: {
        projectid: string;
        version: string;
    };
}
export declare type DebuggeesListRequest = clouddebugger_v2.Params$Resource$Debugger$Debuggees$List;
export declare type DebuggeesBreakpointsDeleteRequest = clouddebugger_v2.Params$Resource$Debugger$Debuggees$Breakpoints$Delete;
export declare type DebuggeesBreakpointsGetRequest = clouddebugger_v2.Params$Resource$Debugger$Debuggees$Breakpoints$Get;
export declare type DebuggeesBreakpointsListRequest = clouddebugger_v2.Params$Resource$Debugger$Debuggees$Breakpoints$List;
export declare type DebuggeesBreakpointsSetRequest = clouddebugger_v2.Params$Resource$Debugger$Debuggees$Breakpoints$Set;
export interface DebuggeesListResponse {
    data: clouddebugger_v2.Schema$ListDebuggeesResponse;
}
export interface DebuggeesBreakpointsDeleteResponse {
    data: clouddebugger_v2.Schema$Empty;
}
export interface DebuggeesBreakpointsGetResponse {
    data: clouddebugger_v2.Schema$GetBreakpointResponse;
}
export interface DebuggeesBreakpointsListResponse {
    data: clouddebugger_v2.Schema$ListBreakpointsResponse;
}
export interface DebuggeesBreakpointsSetResponse {
    data: clouddebugger_v2.Schema$SetBreakpointResponse;
}
/** @fires breakpointHit as soon as any breakpoints are hit */
export interface DebugProxyInterface extends EventEmitter {
    /**
     * Updates the state of all pending breakpoints set by
     * this instance that implements `DebugProxyInterface`.
     *
     * @param block - true to block until breakpoint list changes, otherwise false
     * @fires breakpointHit if any breakpoints were hit since their last checks
     */
    updatePendingBreakpoints(block: boolean): Promise<void>;
    /**
     * @returns project ID for the selected GCP project
     */
    getProjectId(): ProjectId;
    /**
     * @param keyFilename - path to the GCP credentials key file
     * corresponding to the project that is to be debugged
     */
    setProjectByKeyFile(keyFilename?: SourcePath): Promise<void>;
    /**
     * @returns debugger ID for the selected GCP debuggee
     */
    getDebuggerId(): DebuggerId;
    /**
     * @param debuggeeId - ID of the debuggee that is to be debugged
     */
    setDebuggeeId(debuggeeId: DebuggeeId): void;
    /**
     * @returns list of available debuggees for the selected project
     */
    getDebuggees(): Promise<Debuggee[]>;
    /**
     * Retrieves the breakpoint with the given ID.
     *
     * @param breakpointId - ID of the breakpoint to retrieve
     * @returns breakpoint with the given breakpoint ID
     */
    getBreakpoint(breakpointId: BreakpointId): Promise<Breakpoint>;
    /**
     * Removes all breakpoints set by this instance
     * that implements `DebugProxyInterface`.
     */
    removeAllBreakpoints(): Promise<void>;
    /**
     * Removes all breakpoints in the given file that have not been hit.
     *
     * This function is used by UIs that do not send breakpoints incrementally;
     * that is, UIs that send all pending breakpoints together on every update.
     * These semantics can be implemented by clearing them before each update.
     *
     * @param path - path to the file in which breakpoints should be removed
     */
    removePendingBreakpointsForFile(path: SourcePath): Promise<void>;
    /**
     * Removes the breakpoint with the given ID.
     *
     * @param breakpointId - ID of the breakpoint to remove
     */
    removeBreakpoint(breakpointId: BreakpointId): Promise<void>;
    /**
     * Sets a new pending breakpoint.
     *
     * @param breakpointRequest - breakpoint to set
     * @return breakpoint which was successfully set
     */
    setBreakpoint(breakpointRequest: BreakpointRequest): Promise<Breakpoint>;
    /**
     * Retrieves a list of the IDs of all pending or captured snapshots.
     *
     * @param captured - true to get a list of the IDs of all captured snapshots,
     * false to get a list of the IDs of all pending snapshots
     * @returns list of IDs of snapshots of the specified type
     */
    getSnapshotIdList(captured: boolean): BreakpointId[];
}
export declare class DebugProxy extends EventEmitter implements DebugProxyInterface {
    readonly options: Options;
    private wrapper;
    private readonly breakpointInfoMap;
    constructor(options: Options);
    updatePendingBreakpoints(block: boolean): Promise<void>;
    getProjectId(): ProjectId;
    setProjectByKeyFile(keyFilename?: SourcePath): Promise<void>;
    getDebuggerId(): DebuggerId;
    setDebuggeeId(debuggeeId: DebuggeeId): void;
    getDebuggees(): Promise<Debuggee[]>;
    getBreakpoint(breakpointId: BreakpointId): Promise<Breakpoint>;
    removeAllBreakpoints(): Promise<void>;
    removePendingBreakpointsForFile(path: SourcePath): Promise<void>;
    removeBreakpoint(breakpointId: BreakpointId): Promise<void>;
    setBreakpoint(breakpointRequest: BreakpointRequest): Promise<Breakpoint>;
    getSnapshotIdList(captured: boolean): BreakpointId[];
}
