"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2018 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const assert = require("assert");
const events_1 = require("events");
const wrapper_1 = require("./wrapper");
var Action;
(function (Action) {
    Action["CAPTURE"] = "CAPTURE";
    Action["LOG"] = "LOG";
})(Action = exports.Action || (exports.Action = {}));
const hit = Symbol();
const ABORTED_ERROR_CODE = 409; // google.rpc.Code.ABORTED
class DebugProxy extends events_1.EventEmitter {
    constructor(options) {
        super();
        this.options = options;
        this.breakpointInfoMap = new Map();
        this.wrapper = new wrapper_1.Wrapper();
    }
    updatePendingBreakpoints(block) {
        return __awaiter(this, void 0, void 0, function* () {
            // `breakpointList` has all pending breakpoints on the debuggee.
            let breakpointList = [];
            /* debuggees.breakpoints.list times out until the breakpoint list changes.
             * On timeout, it returns the error code google.rpc.Code.ABORTED, and
             * the request should be made again until the breakpoint list changes.
             */
            while (true) {
                try {
                    breakpointList = yield this.wrapper.debuggeesBreakpointsList(block);
                    break;
                }
                catch (error) {
                    if (!error.response || error.response.status !== ABORTED_ERROR_CODE) {
                        throw error;
                    }
                }
            }
            // `pendingBreakpointIdSet` has the IDs of all pending
            // breakpoints set by this instance of `DebugProxy`.
            const pendingBreakpointIdSet = new Set();
            breakpointList.forEach((breakpoint) => {
                const id = breakpoint.id;
                if (this.breakpointInfoMap.has(id)) {
                    assert.strictEqual(this.breakpointInfoMap.get(id)[hit], false);
                    pendingBreakpointIdSet.add(id);
                }
            });
            // A breakpoint in `breakpointInfoMap` but not in `pendingBreakpointIdSet`
            // could be hit since its last check, so check again and update its state.
            const possiblyHitPromiseList = [];
            this.breakpointInfoMap.forEach((info, id) => {
                if (!pendingBreakpointIdSet.has(id)) {
                    possiblyHitPromiseList.push(this.getBreakpoint(id));
                }
            });
            // These breakpoints have either been hit or removed by someone else.
            let hitAny = false;
            const removedBreakpointPromiseList = [];
            // TODO: use https://www.npmjs.com/package/p-limit to rate-limit Promise.all
            const possiblyHitBreakpointList = yield Promise.all(possiblyHitPromiseList);
            possiblyHitBreakpointList.forEach((breakpoint) => {
                if (breakpoint.isFinalState) {
                    assert(this.breakpointInfoMap.has(breakpoint.id));
                    this.breakpointInfoMap.get(breakpoint.id)[hit] = true;
                    hitAny = true;
                }
                else {
                    removedBreakpointPromiseList.push(this.removeBreakpoint(breakpoint.id));
                }
            });
            if (hitAny) {
                this.emit('breakpointHit');
            }
            yield Promise.all(removedBreakpointPromiseList);
        });
    }
    getProjectId() {
        return this.wrapper.getProjectId();
    }
    setProjectByKeyFile(keyFilename) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.wrapper.authorize(keyFilename);
        });
    }
    getDebuggerId() {
        return this.options.debuggerId;
    }
    setDebuggeeId(debuggeeId) {
        this.wrapper.debuggeeId = debuggeeId;
    }
    getDebuggees() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.wrapper.debuggeesList();
        });
    }
    getBreakpoint(breakpointId) {
        if (!this.breakpointInfoMap.has(breakpointId)) {
            throw new Error(`The breakpoint with ID ${breakpointId}, passed into ` +
                '`getBreakpoint`, was not set by this instance of `DebugProxy`.');
        }
        return this.wrapper.debuggeesBreakpointsGet(breakpointId);
    }
    removeAllBreakpoints() {
        return __awaiter(this, void 0, void 0, function* () {
            const promiseList = [];
            this.breakpointInfoMap.forEach((info, id) => {
                promiseList.push(this.wrapper.debuggeesBreakpointsDelete(id));
            });
            this.breakpointInfoMap.clear();
            yield Promise.all(promiseList);
        });
    }
    removePendingBreakpointsForFile(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const promiseList = [];
            this.breakpointInfoMap.forEach((info, id) => {
                if (!info[hit] && info.path === path) {
                    this.breakpointInfoMap.delete(id);
                    promiseList.push(this.wrapper.debuggeesBreakpointsDelete(id));
                }
            });
            yield Promise.all(promiseList);
        });
    }
    removeBreakpoint(breakpointId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.breakpointInfoMap.has(breakpointId)) {
                throw new Error(`The breakpoint with ID ${breakpointId}, passed into ` +
                    '`removeBreakpoint`, was not set by this instance of `DebugProxy`.');
            }
            this.breakpointInfoMap.delete(breakpointId);
            yield this.wrapper.debuggeesBreakpointsDelete(breakpointId);
        });
    }
    setBreakpoint(breakpointRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            const breakpoint = yield this.wrapper.debuggeesBreakpointsSet(breakpointRequest);
            this.breakpointInfoMap.set(breakpoint.id, { [hit]: false, path: breakpoint.location.path });
            return breakpoint;
        });
    }
    getSnapshotIdList(captured) {
        return Array.from(this.breakpointInfoMap)
            .filter(([id, info]) => info[hit] === captured)
            .map(([id, info]) => id);
    }
}
exports.DebugProxy = DebugProxy;
//# sourceMappingURL=index.js.map